"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../lib/errors");
const archive_1 = require("../lib/archive");
const backtrace_service_1 = require("backtrace-service");
const util_1 = require("../lib/util");
const path_1 = require("path");
const log = backtrace_service_1.Logger.getLogger();
const archivesController = (db, service, urlPrefix) => {
    service.app.delete(`${urlPrefix}/:universe_name/:project_name/archives/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { id } = req.params;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send(errors_1.getError(2, "Invalid ID"));
        }
        await archive_1.deleteArchive(parseInt(id, 10));
        return res.status(200).send();
    }));
    service.app.post(`${urlPrefix}/:universe_name/:project_name/archives`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { universe_name, project_name } = req.params;
        const { name, default_commit } = req.body;
        const { archive } = await archive_1.createArchive({
            universe_name,
            project_name,
            name,
            default_commit,
        });
        res.json({ archive });
    }));
    service.app.put(`${urlPrefix}/:universe_name/:project_name/archives/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const id = req.params.id;
        const repo = await db("repositories").first("*").where("id", id);
        if (!repo) {
            return res
                .status(404)
                .send(errors_1.getError(5, `Repository with id ${id} not found`));
        }
        const updateProps = {};
        const { name, default_commit, commit_rules } = req.body;
        if (name && name.trim() !== "") {
            updateProps.name = name.trim();
        }
        if (default_commit && default_commit.trim() !== "") {
            updateProps.default_commit = default_commit.trim();
        }
        await db("repositories").where("id", id).update(updateProps);
        if (commit_rules &&
            Array.isArray(commit_rules) &&
            commit_rules.length > 0) {
            const trimmed = commit_rules
                .map((cr) => cr.trim())
                .filter((cr) => cr.length > 0);
            if (trimmed.length > 0) {
                await db("commit_rules").where({ repository_id: id }).del();
                trimmed.forEach(async (val, idx) => await db("commit_rules").insert({
                    repository_id: id,
                    position: idx,
                    attribute_template: val,
                }));
            }
        }
        const archive = await db("repositories").first("*").where("id", id);
        res.json({ archive });
    }));
    service.app.post(`${urlPrefix}/:universe_name/:project_name/archives/:id/upload`, service.identityManager.authenticateRequest(), util_1.bypassAccessCheck(service, true), util_1.asyncWrapper(async (req, res, _next) => {
        let { id, universe_name, project_name } = req.params;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send(errors_1.getError(2, "Invalid ID"));
        }
        const arch = await db("repositories").first("*").where({
            id,
            universe_name,
            project_name,
        });
        if (!arch) {
            return res
                .status(404)
                .send(errors_1.getError(18, `Archive with id ${id} not found`));
        }
        const { fields, files } = await util_1.parseForm(req);
        if (!fields.secret) {
            return res
                .status(404)
                .send(errors_1.getError(20, "Secret not provided: must include secret in request."));
        }
        if (fields.secret !== arch.archive_secret) {
            return res
                .status(404)
                .send(errors_1.getError(21, `Secret does not match the one on record for ${arch.name}.`));
        }
        const fileName = files.file.name;
        const filePath = files.file.path;
        const tag = fields.tag
            ? fields.tag
            : path_1.basename(fileName, path_1.extname(fileName));
        try {
            await archive_1.addTarToArchive(filePath, tag, parseInt(id, 10));
        }
        catch (e) {
            if (/tag .+ already exists/.test(e.message)) {
                return res
                    .status(400)
                    .send(errors_1.getError(19, "This tag already exists"));
            }
            if (/does not look like a tar archive/.test(e.message)) {
                return res
                    .status(400)
                    .send(errors_1.getError(22, "This does not look like an acceptable archive format"));
            }
            log.error(e);
            return res.status(400).send(errors_1.getError(3));
        }
        return res.status(200).send();
    }));
};
exports.default = archivesController;
//# sourceMappingURL=archives.js.map