"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../lib/errors");
const perforce_1 = require("../lib/perforce");
const util_1 = require("../lib/util");
const viewableColumns = [
    "id",
    "project_name",
    "universe_name",
    "name",
    "host",
    "port",
    "user_name",
    "connection_status",
    "error",
    "changelist_attribute",
    "created_at",
    "updated_at",
];
function authInfoChanged(server, updateProps) {
    return ((updateProps.user_name && updateProps.user_name !== server.user_name) ||
        (updateProps.password && updateProps.password !== server.password) ||
        (updateProps.host && updateProps.host !== server.host) ||
        (updateProps.port && updateProps.port !== server.port));
}
function getSourceCodePaths(paths) {
    const source_code_paths = paths
        .map((scp) => {
        let path = scp.path.trim();
        if (path && path[path.length - 1] !== "/") {
            path += "/";
        }
        return Object.assign(Object.assign({}, scp), { path });
    })
        .filter((scp) => scp.path.length > 0);
    if (source_code_paths &&
        Array.isArray(source_code_paths) &&
        source_code_paths.length > 0) {
        return source_code_paths;
    }
    return [];
}
async function insertSourceCodePaths(paths, id, db) {
    for (const scp of paths) {
        await db("perforce_paths").insert({
            perforce_server_id: id,
            position: scp.position,
            path: scp.path,
        });
    }
}
const perforceController = (db, service, urlPrefix) => {
    service.app.get(`${urlPrefix}/:universeName/:projectName/perforce_servers`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { projectName, universeName } = req.params;
        const perforce_servers = await db("perforce_servers")
            .select(viewableColumns)
            .where({ universe_name: universeName, project_name: projectName });
        for (const server of perforce_servers) {
            const paths = await db("perforce_paths")
                .select("*")
                .where("perforce_server_id", server.id);
            server.source_code_paths = paths;
        }
        res.status(200).send({ perforce_servers });
    }));
    service.app.put(`${urlPrefix}/:universeName/:projectName/perforce_servers/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const id = req.params.id;
        const server = await db("perforce_servers").first("*").where("id", id);
        if (!server) {
            return res
                .status(404)
                .send(errors_1.getError(5, `Perforce server with id ${id} not found`));
        }
        const updateProps = {};
        const { name, host, port, user_name, password, changelist_attribute, } = req.body;
        if (name && name.trim() !== "") {
            updateProps.name = name.trim();
        }
        if (host && host.trim() !== "") {
            updateProps.host = host.trim();
        }
        if (user_name && user_name.trim() !== "") {
            updateProps.user_name = user_name.trim();
        }
        if (password && password.trim() !== "") {
            updateProps.password = password.trim();
        }
        if (changelist_attribute && changelist_attribute.trim() !== "") {
            updateProps.changelist_attribute = changelist_attribute.trim();
        }
        if (updateProps.user_name &&
            updateProps.user_name !== server.user_name &&
            !password) {
            updateProps.password = "";
        }
        if (authInfoChanged(server, updateProps)) {
            try {
                await perforce_1.p4Trust({
                    host,
                    port,
                    password,
                    user_name,
                });
                await perforce_1.getNewTicket({
                    host,
                    port,
                    password,
                    user_name,
                });
            }
            catch (error) {
                await perforce_1.p4Trust({
                    host,
                    port,
                    password,
                    user_name,
                }, true);
                if (error.code === "ETIMEDOUT") {
                    return res.status(400).send(errors_1.getError(13));
                }
                else {
                    return res.status(400).send(errors_1.getError(14, error));
                }
            }
        }
        const source_code_paths = getSourceCodePaths(req.body.source_code_paths);
        if (source_code_paths.length === 0) {
            return res
                .status(422)
                .json(errors_1.getError(16, "source code paths missing"));
        }
        await db("perforce_paths").where({ perforce_server_id: id }).del();
        await insertSourceCodePaths(source_code_paths, id, db);
        await db("perforce_servers")
            .first("id")
            .where("id", id)
            .update(updateProps);
        const paths = await db("perforce_paths").where("perforce_server_id", id);
        const perforce_server = await db("perforce_servers")
            .first(viewableColumns)
            .where("id", id);
        perforce_server.source_code_paths = paths;
        res.status(200).json({ perforce_server });
    }));
    service.app.post(`${urlPrefix}/:universeName/:projectName/perforce_servers`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        if (!perforce_1.p4Available()) {
            return res
                .status(422)
                .json(errors_1.getError(17, 'Unable to locate p4 binary. Please install the Helix p4 command-line client: https://www.perforce.com/downloads/helix-command-line-client-p4. If it is installed and you still receive this error, please specify the path to the p4 command using the P4CMD environment variable, or by adding the path under the "p4cmd" key in sourcecode.conf (a json formatted file which should be placed within your backtrace-source-code installation\'s root folder).'));
        }
        const { projectName, universeName } = req.params;
        let { name, host, user_name, password, changelist_attribute, } = req.body;
        const port = req.body.port;
        if (name && name.trim() !== "") {
            name = name.trim();
        }
        if (host && host.trim() !== "") {
            host = host.trim();
        }
        if (user_name && user_name.trim() !== "") {
            user_name = user_name.trim();
        }
        if (password && password.trim() !== "") {
            password = password.trim();
        }
        if (changelist_attribute && changelist_attribute.trim() !== "") {
            changelist_attribute = changelist_attribute.trim();
        }
        const source_code_paths = getSourceCodePaths(req.body.source_code_paths);
        if (source_code_paths.length === 0) {
            return res
                .status(422)
                .json(errors_1.getError(16, "source code paths missing"));
        }
        let p4AuthTicket;
        await perforce_1.p4Trust({
            host,
            port,
            password,
            user_name,
        });
        try {
            p4AuthTicket = await perforce_1.getNewTicket({
                host,
                port,
                password,
                user_name,
            });
        }
        catch (error) {
            await perforce_1.p4Trust({
                host,
                port,
                password,
                user_name,
            }, true);
            if (error.code === "ETIMEDOUT") {
                return res.status(400).send(errors_1.getError(13));
            }
            else {
                return res.status(400).send(errors_1.getError(14, error));
            }
        }
        const [id] = await db("perforce_servers").insert({
            project_name: projectName,
            universe_name: universeName,
            name,
            host,
            port,
            user_name,
            password,
            changelist_attribute,
            connection_status: "working",
            auth_ticket: p4AuthTicket,
        });
        if (id) {
            await insertSourceCodePaths(source_code_paths, id, db);
        }
        const paths = await db("perforce_paths").where("perforce_server_id", id);
        const perforce_server = await db("perforce_servers")
            .first(viewableColumns)
            .where("id", id);
        perforce_server.source_code_paths = paths;
        return res.status(200).json({ perforce_server });
    }));
    service.app.delete(`${urlPrefix}/:universeName/:projectName/perforce_servers/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { id } = req.params;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res
                .status(422)
                .send(errors_1.getError(2, "Invalid perforce server ID"));
        }
        const server = await db("perforce_servers").first("*").where("id", id);
        if (!server) {
            return res.status(404).send();
        }
        await db("perforce_servers").where("id", id).del();
        const paths = await db("perforce_paths")
            .select("id")
            .where("perforce_server_id", id);
        for (const path of paths) {
            await db("perforce_paths").where("id", path.id).del();
        }
        const { host, port, password, user_name } = server;
        await perforce_1.p4Trust({
            host,
            port,
            password,
            user_name,
        }, true);
        return res.status(200).send();
    }));
};
exports.default = perforceController;
//# sourceMappingURL=perforce.js.map