"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = __importDefault(require("node-fetch"));
const git_routines_1 = require("../lib/git_routines");
const util_1 = require("../lib/util");
const backtrace_service_1 = require("backtrace-service");
const log = backtrace_service_1.Logger.getLogger();
async function setBlame({ coronerUrl, token, universe, project, db, }) {
    const columns = [
        "fingerprint;issues;blame",
        "fingerprint;issues;blame_last_modified",
        "version",
        "application",
        "callstack.files",
        "callstack.modules",
        "callstack.functions",
    ];
    async function getFingerprints(timeRange = [
        Math.floor(Date.now() / 1000) - 2592000,
        Math.floor(Date.now() / 1000),
    ], lastModifiedOlderThan = Math.floor(Date.now() / 1000)) {
        var _a;
        const r = await node_fetch_1.default(coronerUrl + `/api/query?project=${project}`, {
            method: "post",
            headers: {
                "X-Coroner-Token": token,
            },
            body: JSON.stringify({
                group: ["fingerprint"],
                fold: columns.reduce((acc, curr) => {
                    acc[curr] = [["head"]];
                    return acc;
                }, {}),
                filter: [
                    {
                        "fingerprint;issues;blame_last_modified": [
                            ["less-than", lastModifiedOlderThan],
                        ],
                        "callstack.functions": [["regular-expression", ".*"]],
                        timestamp: [
                            ["at-least", timeRange[0]],
                            ["at-most", timeRange[1]],
                        ],
                    },
                ],
                limit: 20,
            }),
        });
        let resJson;
        try {
            resJson = await r.json();
        }
        catch (e) {
            log.error("getFingerprints() Error parsing body: " + (await r.text()));
            throw e;
        }
        if (resJson === null || resJson === void 0 ? void 0 : resJson.error) {
            throw new Error("Error fetching fingerprints from coroner:" +
                JSON.stringify(resJson.error));
        }
        return (_a = resJson === null || resJson === void 0 ? void 0 : resJson.response) === null || _a === void 0 ? void 0 : _a.values;
    }
    async function getBlame(body) {
        const { projectName, universeName, objectName, functionName, directory, fileName, attributes, includeBlame, faultingLine, } = body;
        const fullPath = `${directory}/${fileName}`;
        const surroundingLines = null;
        const obj = {
            fileName,
            fullPath,
            functionName,
            objectName,
            attributes,
        };
        const matchingFile = await git_routines_1.getMatchingFile({
            universeName,
            projectName,
            obj,
            db,
        });
        if (matchingFile === null) {
            log.trace(`getBlame(${universe}, ${project}): No matching file for: ` + fullPath);
            return null;
        }
        if ("repoId" in matchingFile) {
            const { repo, repoId, filePath, commitHash, } = matchingFile;
            const { blame } = await git_routines_1.gitShow({
                repo,
                repoId,
                commitHashUnsanitized: commitHash,
                filePathUnsanitized: filePath,
                faultingLine,
                surroundingLines,
                includeBlame,
                universeName,
                projectName,
            });
            return blame;
        }
        else {
        }
    }
    async function findBlame(callstack, attributes) {
        for (let i = 0; i < callstack.length; i++) {
            const { file, module, func } = callstack[i];
            const [fullPath, line] = file.split(":");
            const pathElements = fullPath.split("/");
            const fileName = pathElements.pop();
            const directory = pathElements.join("/");
            const blame = await getBlame({
                projectName: project,
                universeName: universe,
                objectName: module,
                functionName: func,
                directory,
                fileName,
                attributes: {
                    version: attributes.version,
                    application: attributes.application,
                },
                includeBlame: true,
                faultingLine: line,
            });
            if (blame) {
                const blameAuthor = blame.split("\n")[line];
                if (blameAuthor) {
                    return {
                        blame: blameAuthor,
                        fileName,
                        module,
                        func,
                        line,
                    };
                }
            }
        }
        return Promise.resolve(null);
    }
    async function postBlame(fingerprint, blame) {
        return await node_fetch_1.default(`${coronerUrl}/api/issue`, {
            method: "post",
            headers: {
                accept: "*/*",
                "accept-language": "en-US,en;q=0.9",
                "cache-control": "no-cache",
                "content-type": "application/json",
                pragma: "no-cache",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "cross-site",
                "x-coroner-location": coronerUrl,
                "x-coroner-token": token,
            },
            body: JSON.stringify({
                blame,
                blame_last_modified: Math.floor(Date.now() / 1000),
                fingerprint: [fingerprint],
                project,
            }),
        });
    }
    const fingerprintTimeRange = [
        util_1.daysAgoToSecondsAgo(30),
        util_1.daysAgoToSecondsAgo(0),
    ];
    const blameSetOlderThan = util_1.daysAgoToSecondsAgo(30);
    let groups = await getFingerprints(fingerprintTimeRange, blameSetOlderThan);
    while ((groups === null || groups === void 0 ? void 0 : groups.length) > 0) {
        log.trace(`setBlame(${universe}, ${project}): ` +
            groups.length +
            " fingerprints found");
        for (const group of groups) {
            const [fingerprint, resColumns] = group;
            const attributes = columns.reduce((acc, col, idx) => {
                acc[col] = resColumns[idx];
                return acc;
            }, {});
            const files = JSON.parse(attributes["callstack.files"]).frame;
            const modules = JSON.parse(attributes["callstack.modules"]).frame;
            const functions = JSON.parse(attributes["callstack.functions"]).frame;
            const callstack = files.map((file, idx) => {
                const module = modules[idx];
                const func = functions[idx];
                return {
                    file,
                    module,
                    func,
                };
            });
            const source = await findBlame(callstack, attributes);
            if (source && source.blame) {
                log.trace(`setBlame(${universe}, ${project}): Blame ${source.blame} for Fingerprint ${fingerprint}`);
                await postBlame(fingerprint, source.blame);
            }
            else {
                log.trace(`setBlame(${universe}, ${project}): Blame *BLANK* forFingerprint ${fingerprint}`);
                await postBlame(fingerprint, "");
            }
        }
        groups = await getFingerprints(fingerprintTimeRange, blameSetOlderThan);
    }
}
exports.default = setBlame;
//# sourceMappingURL=blame_setting.js.map