import cache from "../src/lib/cache";
import { gitCmdPath } from "../src/lib/git_routines";
import * as path from "path";
import * as fs from "fs";
import { spawnSync } from "child_process";
import * as rimraf from "rimraf";
import { wrapSpawn, tar } from "../src/lib/util";

export async function usesCache(action: () => Promise<any>) {
  await cache.clearCache();
  const cacheSetSpy = jest.spyOn(cache, "set");
  const cacheGetSpy = jest.spyOn(cache, "get");
  await action();
  expect(cacheGetSpy).toHaveBeenCalledTimes(1);
  // can't use toHaveNthReturnedWith because of promise. must reach in & resolve
  expect(await cacheGetSpy.mock.results[0].value).toBeNull();
  expect(cacheSetSpy).toHaveBeenCalledTimes(1);
  await action();
  expect(cacheGetSpy).toHaveBeenCalledTimes(2);
  expect(cacheSetSpy).toHaveBeenCalledTimes(1);
  expect(await cacheGetSpy.mock.results[1].value).not.toBeNull();
}

const corpusTmp = path.resolve(process.cwd(), "test", "corpusTmp");
if (fs.existsSync(corpusTmp)) {
  rimraf.sync(corpusTmp);
}
fs.mkdirSync(corpusTmp, { recursive: true });

let nextId = 1;

export function gitInit() {
  spawnSync(gitCmdPath(), ["init", nextId.toString()], {
    cwd: corpusTmp,
  });
  const repoPath = path.resolve(corpusTmp, nextId.toString());
  nextId += 1;
  return repoPath;
}

export function appendText(filepath: string, content: string) {
  if (!fs.existsSync(filepath)) {
    fs.mkdirSync(path.dirname(filepath), { recursive: true });
    fs.writeFileSync(filepath, content);
  } else {
    const txt = fs.readFileSync(filepath).toString();
    fs.writeFileSync(filepath, txt + content);
  }
}

export function writeText(filepath: string, content: string) {
  if (!fs.existsSync(filepath)) {
    fs.mkdirSync(path.dirname(filepath), { recursive: true });
  }
  fs.writeFileSync(filepath, content);
}

export function gitAdd(repoPath: string, filename: string) {
  spawnSync(gitCmdPath(), ["add", filename], {
    cwd: repoPath,
  });
}

// commits all tracked files
export function gitCommit(repoPath: string, msg: string) {
  spawnSync(gitCmdPath(), ["commit", "-m", msg], {
    cwd: repoPath,
  });
}

// commits all tracked & untracked files
export function gitCommitAll(repoPath: string, msg: string) {
  gitAdd(repoPath, ".");
  spawnSync(gitCmdPath(), ["commit", "-m", msg], {
    cwd: repoPath,
  });
}

export function gitShow(repoPath: string, filename: string, commit: string) {
  return spawnSync(gitCmdPath(), ["show", `${commit}:${filename}`], {
    cwd: repoPath,
  }).stdout.toString();
}

export function gitListDeletedUnstaged(repoPath: string) {
  return spawnSync(gitCmdPath(), ["diff", "--name-status"], {
    cwd: repoPath,
  }).stdout.toString();
}

export function mktempFile(dir: string | null = null) {
  if (dir) {
    return wrapSpawn("mktemp", ["-p", dir]);
  } else {
    return wrapSpawn("mktemp");
  }
}

export function mktempDir() {
  return wrapSpawn("mktemp", ["-d"]);
}

function zip(inputPath: string, outputPath: string) {
  return wrapSpawn(
    "zip",
    ["-R", outputPath, "*"],
    { timeout: 2000, cwd: inputPath },
    { ignoreStderr: true }
  );
}

export function createTar(ext = "tar") {
  const file = "okay";
  const contents = "hello";

  const dir = mktempDir();
  const filepath = path.resolve(dir, file);
  wrapSpawn("touch", [filepath]);
  writeText(filepath, contents);

  const outdir = mktempDir();
  const tarpath = path.resolve(outdir, `test.${ext}`);
  if (ext == "zip") {
    zip(dir, tarpath);
  } else {
    tar(dir, tarpath);
  }
  return tarpath;
}
